///////////////////////////////////////////////////////////////////////////////
// Model class
//
// Last update: 28/01/2008
///////////////////////////////////////////////////////////////////////////////
import jv.vecmath.PdVector;

public class Model 
{
   protected String[] category;
   protected String[] name;
   protected String   filetype;
   protected String   filename;
   protected boolean  showVertices;
   protected boolean  showEdges;
   protected double   scale = 1.0;
   protected PdVector interest;
   protected PdVector viewDir;
   protected double   dist;
   protected double   roll;
   protected PdVector eye;
   protected double   zVal;
   protected double   visibility;
   protected boolean  enableRotation;
   protected boolean  enablePosition;
   protected double   zMax;
   protected boolean  enableVertexLabels;

   public Model(String[] input_category,
                String[] input_name,
                String   input_filetype,
                String   input_filename,
		boolean  input_showVertices,
		boolean  input_showEdges,
		boolean  input_enableDual,
		boolean  input_enableTruncate)
   {
      category = input_category;
      name = input_name;
      filetype = input_filetype;
      filename = input_filename;
      showVertices = input_showVertices;
      showEdges = input_showEdges;
      
      interest = new PdVector(0, 0, -12.5);
      viewDir = new PdVector(-0.222147, -0.758502, 0.612637);
      dist = 55;
      roll = 79.64958;    
      
      eye = new PdVector(0, 0, -30);
      
      zVal = -20;
      
      visibility = 3.0;
      
      enableRotation = true;
      
      enablePosition = true;
      
      zMax = 0.0;
      
      enableVertexLabels = false;
   };

   public String getCategory(int l)
   {
      return(category[l]);
   };

   public String getName(int l)
   {
      return(name[l]);
   };

   public String getFileType()
   {
      return(filetype);
   };

   public String getFileName()
   {
      return(filename);
   };
   
   public double getScale()
   {
      return(scale);
   };
      
   public boolean isShowingVertices()
   {
      return(showVertices);
   };

   public boolean isShowingEdges()
   {
      return(showEdges);
   };

   public PdVector getInterest()
   {
      return(interest);
   };

   public PdVector getViewDir()
   {
      return(viewDir);
   };

   public double getDist()
   {
      return(dist);
   };
   
   public PdVector getEye()
   {
      return(eye);
   };
   
   public double getZVal()
   {
      return(zVal);
   };   
      
   public double getRoll()
   {
      return(roll);
   };
   
   public double getVisibility()
   {
      return(visibility);
   };
   
   public double getZMax()
   {
      return(zMax);
   };   
   
   public boolean isEnabledRotation()
   {
      return(enableRotation);
   };
   
   public boolean isEnabledPosition()
   {
      return(enablePosition);
   };
   
   public boolean isShowingVertexLabels()
   {
      return(enableVertexLabels);
   };      
         
   public void setScale(double s)
   {
      scale = s;
   };
   
   public void setInterest(double x, double y, double z)
   {
      interest.setEntry(0, x);
      interest.setEntry(1, y);
      interest.setEntry(2, z);      
   };

   public void setViewDir(double x, double y, double z)
   {
      viewDir.setEntry(0, x);
      viewDir.setEntry(1, y);
      viewDir.setEntry(2, z);
   };

   public void setDist(double v)
   {
      dist = v;
   };
   
   public void setRoll(double v)
   {
      roll = v;
   };
   
   public void setEye(double x, double y, double z)
   {
      eye.setEntry(0, x);
      eye.setEntry(1, y);
      eye.setEntry(2, z);      
   };
   
   public void setZVal(double v)
   {
      zVal = v;
   };
   
   public void setVisibility(double v)
   {
      visibility = v;
   };   

   public void enableRotation(boolean v)
   {
      enableRotation = v;
   };
   
   public void enablePosition(boolean v)
   {
      enablePosition = v;
   };
   
   public void setZMax(double v)
   {
      zMax = v;
   };
   
   public void showVertexLabels(boolean v)
   {
      enableVertexLabels = v;
   };   
}; // End of Model 