///////////////////////////////////////////////////////////////////////////////
// PjPerspective class
//
// Last update: 23/01/2008
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Native JAVA imports
///////////////////////////////////////////////////////////////////////////////
import java.awt.Color;
import java.io.*;
import java.net.MalformedURLException;
import java.net.URL;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;


///////////////////////////////////////////////////////////////////////////////
// JavaView imports
///////////////////////////////////////////////////////////////////////////////
import jv.geom.PgElementSet;
import jv.geom.PgPointSet;
import jv.geom.PgPolygon;
import jv.geom.PgPolygonSet;
import jvx.geom.PwCleanMesh;
import jvx.geom.PwTransform;
import jv.loader.PgJvxLoader;
import jv.number.PuDouble;
import jv.object.PsDebug;
import jv.project.PgGeometryIf;
import jv.project.PgJvxSrc;
import jv.project.PjProject;
import jv.project.PvDisplayIf;
import jv.project.PvCameraIf;
import jv.project.PvPickListenerIf;
import jv.project.PvPickEvent;
import jv.vecmath.PdVector;
import jvx.loader.PgOffLoader;


///////////////////////////////////////////////////////////////////////////////
// Poly applet
///////////////////////////////////////////////////////////////////////////////
public class PjPerspective extends PjProject implements PvPickListenerIf
{
   protected int idiom;
   protected URL codebase;
   protected URL fullpath;
   protected double initial_transparency;
   protected boolean enable_choice;

   protected PuDouble transparency;
   protected PuDouble projectionplanesize;
   protected PuDouble projectionplaneposition;
   
   protected int numberOfVertices;
   protected int numberOfEdges;

   protected String category;
   protected String solid;
   protected PgPolygonSet poly;

   protected PvDisplayIf disp; // The PwUnfold needs a reference to the
                               // display in order to work properly.
   protected PvCameraIf cam;   
   
   protected boolean showVertices = true;
   protected boolean showEdges = true;
   protected boolean showElements = false;
   
   protected KeyListener internalKeyListener;   
   

   protected int digitsControl = 0;
   protected boolean isShowingCoordinates = false;
   protected boolean isModeling = false;
   
   protected double x_min = -9.0;
   protected double x_max = 9.0;
   protected double y_min = -9.0;
   protected double y_max = 9.0;
   protected double z_min = -9.0;
   protected double z_max = 9.0;     
   protected PgPolygonSet axes1;
   protected boolean isShowingAxes = false;  
   
   protected double fieldOfView = 0.5;
   
   protected PgElementSet projectionplane;	   
   protected double xMin;
   protected double xMax;
   protected double yMin;
   protected double yMax;
   protected double zMin;
   

   protected PgPolygonSet shadow;
   
   protected double zVal = -20;
   protected double zValMin = -60.0;
   protected double xO = 0.0;
   protected double yO = 0.0;
   protected double zO = -30;	     
   protected double scale = 3.0;
   
   protected PgPointSet eye;
   
   protected PgPolygonSet joininglines;
   

   protected PwTransform m_pwTransform;
   protected PuDouble rot1;
   protected PuDouble rot2;
   protected PuDouble rot3;   
   protected PgElementSet copy;
   
   protected PdVector interest;
   protected PdVector viewDir;
   protected double   dist;
   protected double   roll;   
   
   
   //////////////////////////////////////////////////////////////////////////
   // Default constructor
   //////////////////////////////////////////////////////////////////////////
   public PjPerspective(URL input_codebase,
		                int input_idiom,
		                String input_category,
		                String input_solid,
		                double input_initial_transparency,
		                boolean input_enable_choice)
   {
      super("A Plethora of Polyhedra");

      codebase = input_codebase;
      idiom = input_idiom;
      category = input_category;
      solid = input_solid;
      initial_transparency = input_initial_transparency;
      enable_choice = input_enable_choice;

      if (getClass() == PjPerspective.class)
      {
         init();
      };
   }; // End of PjPoly()

   //////////////////////////////////////////////////////////////////////////
   // Initialize variables
   //////////////////////////////////////////////////////////////////////////
   public void init()
   { 
       poly = new PgPolygonSet(3); 
       
       // Transparency slider
       transparency = new PuDouble("Transparncia:", this);     
       transparency.setDefBounds(0.0, 1.0, 0.01, 0.05);
       transparency.setDefValue(initial_transparency);
       transparency.init();
       transparency.setEnabled(true);
       transparency.setEnabledConfigButton(false);
       
       // Projection plane size slider
       projectionplanesize = new PuDouble("Regio azul:", this);     
       projectionplanesize.setDefBounds(scale, 10.0, 0.01, 0.05);
       projectionplanesize.setDefValue(scale);
       projectionplanesize.init();
       projectionplanesize.setEnabled(true);
       projectionplanesize.setEnabledConfigButton(false);
       
       // Projection plane size slider
       projectionplaneposition = new PuDouble("Regio azul:", this);     
       projectionplaneposition.setDefBounds(-40, 0, 0.01, 0.05);
       projectionplaneposition.setDefValue(zVal);
       projectionplaneposition.init();
       projectionplaneposition.setEnabled(true);
       projectionplaneposition.setEnabledConfigButton(false);
       
       // Axes
       // Axes 1 configuration
       axes1 = new PgPolygonSet(3); // This is a 3D object
       axes1.setNumVertices(6);
       axes1.setNumPolygons(3);
       axes1.setVertex(0, new PdVector(x_min, 0.0, 0.0));
       axes1.setVertex(1, new PdVector(0.0, y_min, 0.0));
       axes1.setVertex(2, new PdVector(0.0, 0.0, z_min));
       axes1.setVertex(3, new PdVector(x_max, 0.0, 0.0));
       axes1.setVertex(4, new PdVector(0.0, y_max, 0.0));
       axes1.setVertex(5, new PdVector(0.0, 0.0, z_max));
       axes1.getVertex(0).setName("");
       axes1.getVertex(1).setName("");
       axes1.getVertex(2).setName("");
       axes1.getVertex(3).setName("x");
       axes1.getVertex(4).setName("y");
       axes1.getVertex(5).setName("z");
       axes1.showVertexLabels(true);
       axes1.setPolygon(0, 0, 3);
       axes1.setPolygon(1, 1, 4);
       axes1.setPolygon(2, 2, 5);
       axes1.setGlobalPolygonColor(Color.BLUE);
       axes1.setGlobalVertexColor(Color.BLUE);
       axes1.setPolygonSize(0, 0.5);
       axes1.setPolygonSize(1, 0.5);
       axes1.setPolygonSize(2, 0.5);
       axes1.showPolygonSizes(true);
       axes1.showPolygonEndArrow(true);  
       
       // Shadow       
	   shadow = new PgPolygonSet(3);
	   
	   // Projection plane
       projectionplane = new PgElementSet(3); // This is a 3D object
       projectionplane.setNumElements(1);
       projectionplane.setNumVertices(4); // Our poly has 8 vertices
       
       // Eye
	   eye = new PgPointSet(3);
	   eye.setNumVertices(1);   
	   eye.setVertex(0, xO, yO, zO);
	   eye.getVertex(0).setName("O");
	   eye.showVertexLabels(true);
	   eye.setLabelColor(0, Color.BLACK);
	   eye.setVertexSize(0, 2.0);
	   eye.showVertexSizes(true);
	   
	   // Joining lines
	   joininglines = new PgPolygonSet(3);
	   joininglines.setGlobalPolygonSize(1.2);
       joininglines.setGlobalPolygonColor(Color.LIGHT_GRAY);  
	   joininglines.showVertices(false);
	   
       // Rotation sliders
       rot1 = new PuDouble("Eixo 1:", this);
       rot1.setDefBounds(-360, 360, 1, 5);
       rot1.setDefValue(0);
       rot1.init();
       rot1.setEnabled(true);
       rot1.setEnabledConfigButton(false);

       rot2 = new PuDouble("Eixo 2:", this);
       rot2.setDefBounds(-360, 360, 1, 5);
       rot2.setDefValue(0);
       rot2.init();
       rot2.setEnabled(true);
       rot2.setEnabledConfigButton(false);

       rot3 = new PuDouble("Eixo 3:", this);
       rot3.setDefBounds(-360, 360, 1, 5);
       rot3.setDefValue(0);
       rot3.init();
       rot3.setEnabled(true);
       rot3.setEnabledConfigButton(false);
       
       // Copy
       copy = new PgElementSet(3);   
       
       // Camera
       interest = new PdVector(3);
       viewDir = new PdVector(3);
   }; // End of init()
   

   //////////////////////////////////////////////////////////////////////////
   // updateShadow()
   //////////////////////////////////////////////////////////////////////////   
   public void updateShadow()
   {
	   shadow.copy(poly);
 	   shadow.showVertices(showVertices);
 	   shadow.setGlobalPolygonSize(1.0);
	   shadow.setGlobalVertexColor(Color.WHITE);

	   zVal = projectionplaneposition.getValue();
	   xO = eye.getVertex(0).getEntry(0);
	   yO = eye.getVertex(0).getEntry(1);
	   zO = eye.getVertex(0).getEntry(2);
	          
       for (int i = 0; i < shadow.getNumVertices(); i++)	       
       {
           double xOld = (poly.getVertices())[i].getEntry(0);
           double yOld = (poly.getVertices())[i].getEntry(1);
           double zOld = (poly.getVertices())[i].getEntry(2);
           
           double xNew = (xOld*(zVal - zO) + xO*(zOld - zVal))/(zOld - zO);  
           double yNew = (yOld*(zVal - zO) + yO*(zOld - zVal))/(zOld - zO);
           double zNew = zVal;
           
           shadow.setVertex(i, new PdVector(xNew, yNew, zNew));
       };

	   if (poly.isShowingVertexLabels() == true)
	   {
		   shadow.getVertex(0).setName("P'");
	   }
       
       shadow.update(shadow);
   }; // End of updateShadow()
   
   
   //////////////////////////////////////////////////////////////////////////
   // updateProjectionPlane()
   //////////////////////////////////////////////////////////////////////////   
   public void updateProjectionPlane()
   {
	   scale = projectionplanesize.getValue();
	   xMin = (poly.getBounds())[0].getEntry(0);
	   yMin = (poly.getBounds())[0].getEntry(1);	
	   xMax = (poly.getBounds())[1].getEntry(0);
	   yMax = (poly.getBounds())[1].getEntry(1);
	   
	   double lx = Math.max(Math.abs(xMin), Math.abs(xMax));
	   double ly = Math.max(Math.abs(yMin), Math.abs(yMax));
	   double l = scale*Math.max(Math.max(lx, ly), 5.0);
	   
       // projectionplane.setVertex(0,  xMax,  yMin, zVal);
       // projectionplane.setVertex(1,  xMax,  yMax, zVal);
       // projectionplane.setVertex(2,  xMin,  yMax, zVal);
       // projectionplane.setVertex(3,  xMin,  yMin, zVal);
       projectionplane.setVertex(0,   l,  -l, zVal);
       projectionplane.setVertex(1,   l,   l, zVal);
       projectionplane.setVertex(2,  -l,   l, zVal);
       projectionplane.setVertex(3,  -l,  -l, zVal);       
       projectionplane.setElement(0, 0, 1, 2, 3);
       projectionplane.showElementColors(true);
       projectionplane.showElements(true);
       projectionplane.showEdges(true);
       projectionplane.setGlobalElementColor(Color.BLUE);
       projectionplane.setTransparency(0.7);
       projectionplane.showTransparency(true);
       
       projectionplane.update(projectionplane);       	   
   }; // End of updateProjectionPlane()
   
   
   //////////////////////////////////////////////////////////////////////////
   // updateJoiningLines()
   //////////////////////////////////////////////////////////////////////////   
   public void updateJoiningLines()
   {
       joininglines.setNumVertices(poly.getNumVertices() + 1);
       joininglines.setNumPolygons(poly.getNumVertices());
       joininglines.setVertex(0, eye.getVertex(0));
       for (int i = 0; i < poly.getNumVertices(); i++)
       {
    	   joininglines.setVertex(i + 1, poly.getVertex(i));
    	   joininglines.setPolygon(i, 0, i + 1);
       }
       
       joininglines.update(joininglines);       	   
   }; // End of updateJoiningLines()   


   //////////////////////////////////////////////////////////////////////////
   // Load off file
   //////////////////////////////////////////////////////////////////////////
   public void loadGeometryFromFile(Model object) 
   {				
	  String filename = object.getFileName();
	  String filetype = object.getFileType();	  

      String pathname = codebase.toExternalForm() + filename + "." + filetype;      
      try
      {
         fullpath = new URL(pathname);
         java.io.InputStream r = fullpath.openStream();
         BufferedReader in = new BufferedReader(new InputStreamReader(r));
         if(in == null)
         {
            PsDebug.warning("could not open file  " + pathname + "...");
            System.out.println("could not open file  " + pathname + "...");
         }
	     else
         {
            // Loading polyhedron
	    	PgJvxSrc[] geometries; 
	    	if (filetype.equals("off"))
	    	{
	    	   PgOffLoader offloader = new PgOffLoader(); 
               offloader.load(in);            
               geometries = offloader.getGeometries();
               poly.setJvx(geometries[0]);
	    	}
	    	if (filetype.equals("jvx"))
	    	{
	    	   PgJvxLoader jvxloader = new PgJvxLoader(); 
               jvxloader.load(in);            
               geometries = jvxloader.getGeometries();
               poly.setJvx(geometries[0]);
	    	}
	    	
	        // Configuring the polyhedron	        
            poly.setTransparency(transparency.getValue());
            poly.showVertices(showVertices);
            if (transparency.getValue() > 0.05)
            {
               poly.showTransparency(true);
            }
            else
            {
               poly.showTransparency(false);	
            }
            poly.showPolygons(showEdges);
            poly.showVertexOutline(true);
            // poly.setGlobalEdgeColor(Color.BLACK);
            poly.setGlobalVertexColor(Color.YELLOW);
            poly.showVertexColors(true);
            // poly.showEdgeColors(true);
			poly.setGlobalPolygonSize(1.0);            

            PwCleanMesh.identifyVertices(poly, 1E-010D);
            
            // Scale
            PwTransform.scale(poly, object.getScale());
            
			// Setup visualization
			showVertices = object.isShowingVertices();
			showEdges = object.isShowingEdges();
			poly.showVertices(showVertices);
            
            // Add because key events are now internally handled			
            poly.showVertexLabels(object.isShowingVertexLabels());
            if (object.isShowingVertexLabels() == true)
            {
            	poly.getVertex(0).setName("P");
            }
            
            // if (poly.getDiameter()/2.0 > 1.5)
            {                
                PdVector[] bb = poly.getBounds();
                double sf = 1.2;
                x_max = sf*Math.max(Math.abs(bb[0].getEntry(0)), Math.abs(bb[1].getEntry(0)));
                x_min = -x_max;
                y_max = sf*Math.max(Math.abs(bb[0].getEntry(1)), Math.abs(bb[1].getEntry(1)));
                y_min = -y_max;
                z_max = sf*Math.max(Math.abs(bb[0].getEntry(2)), Math.abs(bb[1].getEntry(2)));
                z_min = -z_max;
                axes1.setVertex(0, new PdVector(x_min, 0.0, 0.0));
                axes1.setVertex(1, new PdVector(0.0, y_min, 0.0));
                axes1.setVertex(2, new PdVector(0.0, 0.0, z_min));
                axes1.setVertex(3, new PdVector(x_max, 0.0, 0.0));
                axes1.setVertex(4, new PdVector(0.0, y_max, 0.0));
                axes1.setVertex(5, new PdVector(0.0, 0.0, z_max));
                axes1.getVertex(0).setName("");
                axes1.getVertex(1).setName("");
                axes1.getVertex(2).setName("");
                axes1.getVertex(3).setName("x");
                axes1.getVertex(4).setName("y");
                axes1.getVertex(5).setName("z");
                axes1.showVertexLabels(true);
                axes1.setPolygon(0, 0, 3);
                axes1.setPolygon(1, 1, 4);
                axes1.setPolygon(2, 2, 5);
                axes1.setGlobalPolygonColor(Color.BLUE);
                axes1.setGlobalVertexColor(Color.BLUE);
                axes1.setGlobalVertexSize(0.0);
                axes1.setPolygonSize(0, 0.5);
                axes1.setPolygonSize(1, 0.5);
                axes1.setPolygonSize(2, 0.5);
                axes1.showPolygonSizes(true);
                axes1.showPolygonEndArrow(true);
                axes1.update(axes1);
            };
            
            poly.update(poly);
            
            // Eye and projection plane
            eye.setVertex(0, object.getEye());
            eye.update(eye);
            zVal = object.getZVal();
            projectionplanesize.setValue(object.getVisibility());
            projectionplaneposition.setValue(zVal);
          
            // Reset coordinates display
            isShowingCoordinates = false;
            
            // Update projection plane
            updateProjectionPlane();
            
            // Update shadow
            updateShadow();
            
            // Update joining lines
            updateJoiningLines();
            
            // Reset rotation sliders
			rot1.setValue(0.0);
			rot2.setValue(0.0);
			rot3.setValue(0.0);		
			
			// Copy of poly
			copy.copy(poly);		
			
			interest = object.getInterest();
			viewDir = object.getViewDir();
			dist = object.getDist();
			roll = object.getRoll();
						
			// Enable rotation
			rot1.setEnabled(object.isEnabledRotation());
			rot2.setEnabled(object.isEnabledRotation());
			rot3.setEnabled(object.isEnabledRotation());
			
			// Enable position
			projectionplaneposition.setBounds(-40, object.getZMax());
			projectionplaneposition.setDefBounds(-40, object.getZMax(), 0.01, 0.05);
			projectionplaneposition.setEnabled(object.isEnabledPosition());
         }
      }
      catch(MalformedURLException exp)
      {
         System.out.println(exp.toString());
      }
      catch(IOException exp)
      {
         System.out.println(exp.toString());
      }
            
   }; // End of loadGeometryFromFile() 
   

   //////////////////////////////////////////////////////////////////////////
   // Update camera
   //////////////////////////////////////////////////////////////////////////   
   public void updateCamera()
   {
		if (cam != null)
		{		
	       cam.setInterest(interest);
	       cam.setViewDir(viewDir);
	       cam.setDist(dist);
	       cam.setRoll(roll);
	       cam.update(cam);  	
	       // disp.fit();
	       cam.setDist(dist);
	       cam.update(cam);
		}	   
   }; // End of updateCamera()


   //////////////////////////////////////////////////////////////////////////
   // Things to do when the class is started
   //////////////////////////////////////////////////////////////////////////
   public void start()
   {	   
      super.start();

      // Necessary to apply PwWorkshop operations
      PwCleanMesh.identifyVertices(poly, 1E-010D);

      addGeometry(poly);
      addGeometry(projectionplane);
      addGeometry(shadow);
      addGeometry(eye);
      addGeometry(joininglines);
      selectGeometry(eye);

      update(this);

      disp = getDisplay(); // The PwUnfold needs a reference to the
      disp.setAutoCenter(false);      
            
      // display in order to work properly.
      // disp.fit();
  	  cam = disp.getCamera();
  	  
      // Setup camera
      updateCamera();      
  	  
      // Key events
      internalKeyListener = new KeyAdapter() 
      {
          public void keyPressed(KeyEvent keyEvent) 
          {
        	  if (keyEvent.getKeyCode() == KeyEvent.VK_1)
        	  {        	  
	        	  if (poly.isShowingVertexLabels() == true)
	        	  {
	        	     poly.showVertexLabels(false);
	        	  }
	        	  else
	        	  {
	         	     poly.showVertexLabels(true);
	        		 if (isModeling == true)
	        		 {
                        if (isShowingCoordinates == true)
                        {
                           updateCoordinatesLabels();
                        }
	        		 }
	         	     
	        	  }
	              poly.update(poly);
        	  };
        	  
        	  if (keyEvent.getKeyCode() == KeyEvent.VK_2)
        	  {
      			  if ((poly.isShowingVertexLabels() == true) && (isShowingCoordinates == true))
       			  {
          		     digitsControl++;
           		     digitsControl = digitsControl % 8;
       			  }
      			  else
      			  {
      				  isShowingCoordinates = true;
      			  };        		  
        		  
        		  long nd = digitsControl + 1;
        		  long p = (long) Math.pow(10, nd);        		  
        		  long d;
        		  for (int i = 0; i < poly.getNumVertices(); i++)
        		  {
        			  PdVector v = poly.getVertex(i);
        			  String x = Double.toString(Math.round(v.getEntry(0)*p)/(p*1.0));
        			  d = x.length() - x.indexOf('.');
        			  if (nd >= d)
        			  {
        				  for (int j = 0; j <= nd - d; j++)
        				  {
        					  x = x + "0";
        				  }
        			  }
        			  
        			  String y = Double.toString(Math.round(v.getEntry(1)*p)/(p*1.0));
        			  d = y.length() - y.indexOf('.');
        			  if (nd >= d)
        			  {
        				  for (int j = 0; j <= nd - d; j++)
        				  {
        					  y = y + "0";
        				  }
        			  }
        			  
        			  String z = Double.toString(Math.round(v.getEntry(2)*p)/(p*1.0));
        			  d = z.length() - z.indexOf('.');
        			  if (nd >= d)
        			  {
        				  for (int j = 0; j <= nd - d; j++)
        				  {
        					  z = z + "0";
        				  }
        			  }
        			  
        			  v.setName("(" + x + ", " + y + "," + z + ")");
        		  };      			  
        		  poly.update(poly);
        	  };
        	  
        	  if (keyEvent.getKeyCode() == KeyEvent.VK_3)
        	  {
        		  isShowingCoordinates = false;
        		  for (int i = 0; i < poly.getNumVertices(); i++)
        		  {
        			  poly.getVertex(i).setName(Integer.toString(i));
        		  };
        		  poly.update(poly);
        	  };
        	  
        	  if (keyEvent.getKeyCode() == KeyEvent.VK_4)
        	  {
        		  if (isShowingAxes == false)
        		  {
        			  disp.addGeometry(axes1);
        			  isShowingAxes = true;
        		  }
        		  else
        		  {
        			  disp.removeGeometry(axes1);
        			  isShowingAxes = false;        			  
        		  };
        		  disp.update(disp);
        	  };
        	  
        	  if (keyEvent.getKeyCode() == KeyEvent.VK_5)
        	  {
        		  if (disp.isShowingBndBox() == false)
        		  {
        			  disp.showBndBox(true);
        		  }
        		  else
        		  {
        			  disp.showBndBox(false);        			  
        		  };
        	  };
        	  
        	  if (keyEvent.getKeyCode() == KeyEvent.VK_6)
        	  {
        		  fieldOfView -= 0.05;
        		  if (fieldOfView <= 0.0)
        		  {
        			  fieldOfView = 0.0;
        		  };
        		  cam.setFieldOfView(fieldOfView);
        		  cam.update(cam);
        	  };        	  
        	  
        	  if (keyEvent.getKeyCode() == KeyEvent.VK_7)
        	  {
        		  fieldOfView += 0.05;
        		  if (fieldOfView >= 2.0)
        		  {
        			  fieldOfView = 2.0;
        		  };
        		  cam.setFieldOfView(fieldOfView);
        		  cam.update(cam);
        	  };        	  
        	          	          	  
        	  
        	  if ((keyEvent.getKeyCode() == KeyEvent.VK_0) || (keyEvent.getKeyCode() == KeyEvent.VK_8))
        	  {        	  
        		 fieldOfView = 0.5;
                 cam.setFieldOfView(fieldOfView);
                 cam.update(cam);
                 disp.fit();
        	  };        	  
          }
       };
       disp.addKeyListener(internalKeyListener);
   }; // End of start()


   /////////////////////////////////////////////////////////////////////////
   // Update the class whenever a child has changed. Method is usually
   // invoked from the children.
   /////////////////////////////////////////////////////////////////////////
   public boolean update(Object event)
   {
      if (event == null)
      {
         return(true);
      };
      
      // Solid transparency
      if (event == transparency)
      {
          poly.setTransparency(transparency.getValue());

          if (transparency.getValue() > 0.05)
          {
             poly.showTransparency(true);
          }
          else
          {
             poly.showTransparency(false);	
          }
          
          poly.update(poly);
          
          return(true);
      };
      
      // Projection plane size
      if (event == projectionplanesize)
      {
    	  scale = projectionplanesize.getValue();
          
          updateProjectionPlane();
          
          return(true);
      };
      
      // Projection plane position
      if (event == projectionplaneposition)
      {
    	  double value = projectionplaneposition.getValue();
          if (eye.getVertex(0).getEntry(2) > value)
          {
        	  double x = eye.getVertex(0).getEntry(0);
        	  double y = eye.getVertex(0).getEntry(1);
        	  eye.setVertex(0, new PdVector(x, y, value));
          }
          eye.update(eye);
          zVal = value;
          updateShadow();
          updateJoiningLines();
          updateProjectionPlane();
          
          return(true);
      };      
      
	  // Rotation
	  if (event == rot1 || event == rot2 || event == rot3) 
	  {
		  double angle1 = rot1.getValue() * Math.PI / 180.0;
		  double angle2 = rot2.getValue() * Math.PI / 180.0;
		  double angle3 = rot3.getValue() * Math.PI / 180.0;

		  poly.copy(copy);
		  poly.showVertices(showVertices);

		  PwTransform.rotate(poly, angle1, 0);
		  PwTransform.rotate(poly, angle2, 1);
		  PwTransform.rotate(poly, angle3, 2);

		  poly.update(poly);
		  updateShadow();
		  updateJoiningLines();

		  return (true);
	  };
      

      return(super.update(event));
   }; // End of update()
   
   protected void updateCoordinatesLabels()
   {
       if ((isShowingCoordinates == true) && (poly.isShowingVertexLabels() == true))
 	   {
   		  long nd = digitsControl + 1;
		  long p = (long) Math.pow(10, nd);        		  
		  long d;
		  for (int i = 0; i < poly.getNumVertices(); i++)
		  {
			  PdVector v = poly.getVertex(i);
			  String x = Double.toString(Math.round(v.getEntry(0)*p)/(p*1.0));
			  d = x.length() - x.indexOf('.');
			  if (nd >= d)
			  {
				  for (int j = 0; j <= nd - d; j++)
				  {
					  x = x + "0";
				  }
			  }
			  
			  String y = Double.toString(Math.round(v.getEntry(1)*p)/(p*1.0));
			  d = y.length() - y.indexOf('.');
			  if (nd >= d)
			  {
				  for (int j = 0; j <= nd - d; j++)
				  {
					  y = y + "0";
				  }
			  }
			  
			  String z = Double.toString(Math.round(v.getEntry(2)*p)/(p*1.0));
			  d = z.length() - z.indexOf('.');
			  if (nd >= d)
			  {
				  for (int j = 0; j <= nd - d; j++)
				  {
					  z = z + "0";
				  }
			  }
			  
			  v.setName("(" + x + ", " + y + "," + z + ")");
		   };
		   
		   poly.update(poly);
	   };	   
   }; // End of updateCoordinateLabels()

   
   /***************************************************************************
    ***************************************************************************
    **  Begin of PvPickListenerIf Implementation
    ***************************************************************************
    ***************************************************************************/

    //////////////////////////////////////////////////////////////////////////
    // The name of a listeners allows the display to issue verbal debug
    // messages.
    //////////////////////////////////////////////////////////////////////////
    public String getName()
    {
       return("EarthPicker");
    }; // End of getName()

    //////////////////////////////////////////////////////////////////////////
    // Get a location in the display with 2d display and 3d world coordinates.
    // Method is called when display is in mode PvDisplayIf.MODE_DISPLAY_PICK.
    //////////////////////////////////////////////////////////////////////////
    public void pickDisplay(PvPickEvent pos)
    {
       // PsDebug.message("pickDisplay() called ...");
       // System.out.println("pickDisplay() called ...");
    }; // End of pickDisplay()

    //////////////////////////////////////////////////////////////////////////
    // Drag a location in the display with 2d display and 3d world coordinates.
    // Method is called when display is in mode PvDisplayIf.MODE_DISPLAY_PICK.
    //////////////////////////////////////////////////////////////////////////
    public void dragDisplay(PvPickEvent pos)
    {
       // PsDebug.message("pickDisplay() called ...");
       // System.out.println("pickDisplay() called ...");
    }; // End of dragDisplay()

    //////////////////////////////////////////////////////////////////////////
    // Pick an arbitrary point on a geometry, point may lie inside an element.
    // Method is called when display is in mode PvDisplayIf.MODE_INITIAL_PICK
    // or if temporarily the i-key is pressed, and any pixel in the display is
    // picked.
    //////////////////////////////////////////////////////////////////////////
    public void pickInitial(PvPickEvent pos)
    {
       // PsDebug.message("pickInitial() called ...");
       // System.out.println("pickInitial() called ...");
       
    }; // End of pickInitial()

    //////////////////////////////////////////////////////////////////////////
    // Drag an arbitrary point along a geometry, point may lie inside an
    // element. Method is called when display is in mode
    // PvDisplayIf.MODE_INITIAL_PICK or if temporarily the i-key is pressed,
    // and any pixel in the display is dragged.
    //////////////////////////////////////////////////////////////////////////
    public void dragInitial(PvPickEvent pos)
    {
       
    }; // End of dragInitial()

    //////////////////////////////////////////////////////////////////////////
    // Get a picked vertex of a geometry.
    // Method is called when display is in mode PvDisplayIf.MODE_PICK
    // or if temporarily the p-key is pressed, and a vertex picked.
    //////////////////////////////////////////////////////////////////////////
    public void pickVertex(PgGeometryIf geom, int index, PdVector vertex)
    {
        // PsDebug.message("pickVertex() called ...");
        // System.out.println("pickVertex() called ...");
    }; // End of dragInitial();

    //////////////////////////////////////////////////////////////////////////
    // Method is called when display is in mode PvDisplayIf.MODE_PICK
    // or if temporarily the p-key is pressed, and a vertex dragged.
    //////////////////////////////////////////////////////////////////////////
    public void dragVertex(PgGeometryIf a_geom, int index, PdVector vertex)
    {
       // PsDebug.message("dragVertex() called ...");
       // System.out.println("dragVertex() called ...");
    	
		   xO = vertex.getEntry(0);
		   yO = vertex.getEntry(1);
		   zO = vertex.getEntry(2);
		   
		   if (zO > zVal)
		   {
			   zO = zVal;
		   }
		   
		   if (zO < zValMin)
		   {
			   zO = zValMin;
		   }
		   
		   a_geom.setVertex(0, new PdVector(xO, yO, zO));

           updateShadow();
           updateJoiningLines();
    }; // End of dragVertex()

    //////////////////////////////////////////////////////////////////////////
    // Mark a set of vertices of a geometry within a given bounding box.
    // Method is called when display is in mode PvDisplayIf.MODE_MARK
    // or if temporarily the m-key is pressed, and a rectangle is drawn.
    //////////////////////////////////////////////////////////////////////////
    public void markVertices(PvPickEvent markBox)
    {
       // PsDebug.message("markVertices() called ...");
       // System.out.println("markVertices() called ...");
    }; // End of markVertices()

    //////////////////////////////////////////////////////////////////////////
    // Unmark a set of vertices of a geometry within a given bounding box.
    // Method is called when display is in mode PvDisplayIf.MODE_UNMARK
    // or if temporarily the u-key is pressed, and a rectangle is drawn.
    //////////////////////////////////////////////////////////////////////////
    public void unmarkVertices(PvPickEvent markBox)
    {
       // PsDebug.message("unmarkVertices() called ...");
       // System.out.println("unmarkVertices() called ...");
    }; // End of unmarkVertices()

   /***************************************************************************
    ***************************************************************************
    **  End of PvPickListenerIf Implementation
    ***************************************************************************
    ***************************************************************************/

}; // End of PjPerspective classs
