///////////////////////////////////////////////////////////////////////////////
// PjPerspective Interface class
//
// Last update: 13/01/2008
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Native JAVA imports
///////////////////////////////////////////////////////////////////////////////
import java.awt.Checkbox;
import java.awt.Choice;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.TextField;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

///////////////////////////////////////////////////////////////////////////////
// JavaView imports
///////////////////////////////////////////////////////////////////////////////
import jv.object.PsPanel;
import jv.object.PsUpdateIf;
import jv.project.PjProject_IP;


///////////////////////////////////////////////////////////////////////////////
// Info panel for graph tomography applet.
///////////////////////////////////////////////////////////////////////////////
public class PjPerspective_IP extends PjProject_IP implements ActionListener, ItemListener
{
   protected PjPerspective m_pjPerspective;

   protected Choice poly_category;
   protected Choice poly_name;

   protected Panel m_aPanel; // Transparency panel
   protected Panel m_sPanel; // Projection plane size panel
   protected Panel m_pPanel; // Projection plane size panel

   protected Panel m_rot1Panel;
   protected Panel m_rot2Panel;
   protected Panel m_rot3Panel;   
   
   protected Checkbox viewEdgesCB;
   protected Checkbox viewVerticesCB;
   protected Checkbox viewJoiningLinesCB;   

   protected int idiom = 0;
   protected String[] title_trans = {"Projees em Perspectiva",
                                     "Perspective Projections"};

   protected String[] vis_transparency_trans = {"Transparncia:",
		                                        "Transparency:"};
   protected String[] vis_projectionplanesize_trans = {"Visibilidade:",
                                                       "Visibility:"};
   protected String[] vis_projectionplaneposition_trans = {"Posio:",
                                                           "Position:"};      
   protected String[] vis_vertices_trans = {"Vrtices",
                                            "Vertices"};
   protected String[] vis_edges_trans = {"Exibir arestas", "Show edges"};
   protected String[] vis_joininglines_trans = {"Segmentos auxiliares", "Auxiliary segments"};   
   protected String[] vis_panel_trans = {"Exibir", "Show"};

   protected String[] ppl_panel_trans = {"Plano de projeo", "Projection plane"};   
   protected String[] rot_panel_trans = {"Rotao", "Rotate"};   
   protected String[] rot1_trans = {"Eixo 1:", "Yaw:"};   
   protected String[] rot2_trans = {"Eixo 2:", "Pitch:"};
   protected String[] rot3_trans = {"Eixo 3:", "Roll:"};        

   protected String[] object_trans =
                {"Objetos de estudo", "Objects of study"};   
   protected String[] platonic_trans =
                {"Slidos Platnicos", "Platonic Solids"};
   protected String[] animal_trans =
                {"Animais", "Animals"};

   protected Category object;   
   protected Category platonic;
   protected Category animal;

   protected Model model;     
   
   ///////////////////////////////////////////////////////////////////////////
   // Default constructor
   ///////////////////////////////////////////////////////////////////////////
   public PjPerspective_IP()
   {
      super();

      if (getClass() == PjPerspective_IP.class)
      {
         init();
      };
   }; // End of PjPerspective_IP()


   //////////////////////////////////////////////////////////////////////////
   // Setup the main control panel
   //////////////////////////////////////////////////////////////////////////
   public void init()
   {
      super.init();

      // Title
      addTitle(title_trans[idiom]);

      add(new Label(" "));
      // {
      //	 Label l = new Label(title_trans[idiom], Label.CENTER);
      //     l.setFont(new Font("TimesRoman", Font.BOLD, 14));      
      //  add(l);
      // }      
      
      poly_category = new Choice();
      add(poly_category);
      poly_category.addItemListener(this);

      poly_name = new Choice();
      add(poly_name);
      poly_name.addItemListener(this);

      add(new Label(" "));
   }; // End of init()


   //////////////////////////////////////////////////////////////////////////
   // Setup parent and add the level slider
   //////////////////////////////////////////////////////////////////////////
   public void setParent(PsUpdateIf parent)
   {
      super.setParent(parent);
      m_pjPerspective = (PjPerspective) parent;
      idiom = m_pjPerspective.idiom;
      
      // Title
      // setTitle(title_trans[idiom]);

      // Category and name of the solid
      poly_category.add(object_trans[idiom]);
      poly_category.add(platonic_trans[idiom]);
      poly_category.add(animal_trans[idiom]);
      
      poly_category.select(m_pjPerspective.category);
      buildDataBase();
      buildChoice(m_pjPerspective.category);
      poly_name.select(m_pjPerspective.solid);
      loadModel(m_pjPerspective.category, m_pjPerspective.solid);
      m_pjPerspective.updateCamera();      

      poly_category.enable(m_pjPerspective.enable_choice);
      poly_name.enable(m_pjPerspective.enable_choice);

      
      // Visualization panel
      
      PsPanel visPanel = new PsPanel();
      
      visPanel.addLine(1);
      
      {
    	  Label l = new Label(vis_panel_trans[idiom], Label.CENTER);
          l.setFont(new Font("TimesRoman", Font.BOLD, 14));      
    	  visPanel.add(l);
      }
      
      Panel p = new Panel();
      
      viewJoiningLinesCB = new Checkbox(vis_joininglines_trans[idiom]);
      viewJoiningLinesCB.addItemListener(this);
      viewJoiningLinesCB.setState(true);
      p.add(viewJoiningLinesCB);      
      
      p.setLayout(new GridLayout(1, 2));
      viewVerticesCB = new Checkbox(vis_vertices_trans[idiom]);
      viewVerticesCB.addItemListener(this);
      viewVerticesCB.setState(true);
      p.add(viewVerticesCB);

      viewEdgesCB = new Checkbox(vis_edges_trans[idiom]);
      viewEdgesCB.addItemListener(this);
      viewEdgesCB.setState(true);
      // visPanel.add(viewEdgesCB);

      // m_aPanel = new Panel();
      // m_aPanel.setLayout(new GridLayout(1, 1));
      // visPanel.add(m_aPanel);
      // m_aPanel.removeAll();
      // m_pjPerspective.transparency.setName(vis_transparency_trans[idiom]);
      // m_aPanel.add(m_pjPerspective.transparency.newInspector(PsPanel.INFO_EXT));
      //
      // visPanel.addLine(1);
      
      visPanel.add(p);
      
      // visPanel.addLine(1);

      add(visPanel);
      

      // Projection plane panel
      PsPanel pplPanel = new PsPanel();

      pplPanel.addLine(1);
      {
    	  Label l = new Label(ppl_panel_trans[idiom], Label.CENTER);
          l.setFont(new Font("TimesRoman", Font.BOLD, 14));      
    	  pplPanel.add(l);
      }
      
      m_sPanel = new Panel();
      m_sPanel.setLayout(new GridLayout(1, 1));
      pplPanel.add(m_sPanel);
      m_sPanel.removeAll();
      m_pjPerspective.projectionplanesize.setName(vis_projectionplanesize_trans[idiom]);
      m_sPanel.add(m_pjPerspective.projectionplanesize.newInspector(PsPanel.INFO_EXT));
      
      m_pPanel = new Panel();
      m_pPanel.setLayout(new GridLayout(1, 1));
      pplPanel.add(m_pPanel);
      m_pPanel.removeAll();
      m_pjPerspective.projectionplaneposition.setName(vis_projectionplaneposition_trans[idiom]);
      m_pPanel.add(m_pjPerspective.projectionplaneposition.newInspector(PsPanel.INFO_EXT));           

      pplPanel.addLine(1);
      
      add(pplPanel);

      // Rotate panel
      PsPanel rotPanel = new PsPanel();
      
      {
    	  Label l = new Label(rot_panel_trans[idiom], Label.CENTER);
          l.setFont(new Font("TimesRoman", Font.BOLD, 14));      
    	  rotPanel.add(l);
      }
            
      m_rot1Panel = new Panel();
      m_rot1Panel.setLayout(new GridLayout(1, 1));
      rotPanel.add(m_rot1Panel);
      m_rot1Panel.removeAll();
      m_pjPerspective.rot1.setName(rot1_trans[idiom]);
      m_rot1Panel.add(m_pjPerspective.rot1.newInspector(PsPanel.INFO_EXT));

      m_rot2Panel = new Panel();
      m_rot2Panel.setLayout(new GridLayout(1, 1));
      rotPanel.add(m_rot2Panel);
      m_rot2Panel.removeAll();
      m_pjPerspective.rot2.setName(rot2_trans[idiom]);
      m_rot2Panel.add(m_pjPerspective.rot2.newInspector(PsPanel.INFO_EXT));

      m_rot3Panel = new Panel();
      m_rot3Panel.setLayout(new GridLayout(1, 1));
      rotPanel.add(m_rot3Panel);
      m_rot3Panel.removeAll();
      m_pjPerspective.rot3.setName(rot3_trans[idiom]);
      m_rot3Panel.add(m_pjPerspective.rot3.newInspector(PsPanel.INFO_EXT));
      
      // rotPanel.addLine(1);
      
      add(rotPanel);
      
      super.setParent(parent);
   }; // End of setParent()
   


   //////////////////////////////////////////////////////////////////////////
   // Load model
   //////////////////////////////////////////////////////////////////////////
   public void loadModel(String category, String solid)
   {
	   
	  if (category.equals(object_trans[idiom]))
	  {
		  for (int i = 0; i < object.getNumberOfModels(); i++)
		  {
			  if (object.getModel(i).getName(idiom).equals(solid))
			  {
				  model = object.getModel(i);
				  break;
			  }
		  }
	  }
	   
	  if (category.equals(platonic_trans[idiom]))
	  {
		  for (int i = 0; i < platonic.getNumberOfModels(); i++)
		  {
			  if (platonic.getModel(i).getName(idiom).equals(solid))
			  {
				  model = platonic.getModel(i);
				  break;
			  }
		  }
	  }

	  if (category.equals(animal_trans[idiom]))
	  {
		  for (int i = 0; i < animal.getNumberOfModels(); i++)
		  {
			  if (animal.getModel(i).getName(idiom).equals(solid))
			  {
				  model = animal.getModel(i);
				  break;
			  }
		  }
	  }	  
	
		  
      m_pjPerspective.showVertices = model.isShowingVertices();
      m_pjPerspective.showEdges = model.isShowingEdges();
      
      if (viewVerticesCB != null)
      {
         viewVerticesCB.setState(model.isShowingVertices());
      };
      
      m_pjPerspective.loadGeometryFromFile(model);
   }; // End of loadModel()

   //////////////////////////////////////////////////////////////////////////
   // Update the panel whenever the parent has changed somewhere else
   //////////////////////////////////////////////////////////////////////////
   public boolean update(Object  event)
   {
      if (m_pjPerspective == event)
      {
         // Fill here!

         return(true);
      };

      return(super.update(event));
   }; // End of update()


   //////////////////////////////////////////////////////////////////////////
   // Handle action events invoked from text fields
   //////////////////////////////////////////////////////////////////////////
   public void actionPerformed(ActionEvent event)
   {
      if (m_pjPerspective == null)
      {
          return;
      };

      // Object source = event.getSource();

      // Fill here!

   }; // End of actionPerformed()


   //////////////////////////////////////////////////////////////////////////
   // Handle action events invoked from itens
   //////////////////////////////////////////////////////////////////////////
   public void itemStateChanged(ItemEvent event)
   {
       if (m_pjPerspective == null)
       {
          return;
       };

       Object source = event.getSource();

       //////////////////////////////////////////////////////////////////////
       // Names
       //////////////////////////////////////////////////////////////////////
       if (source == poly_category)
       {
          String category = poly_category.getSelectedItem();
          buildChoice(category);
          poly_name.select(0);
          loadModel(category, poly_name.getSelectedItem());          

          // m_pjPerspective.disp.fit();
          m_pjPerspective.updateCamera();
       }

       if (source == poly_name)
       {
          loadModel(poly_category.getSelectedItem(),
        		    poly_name.getSelectedItem());
          
          // m_pjPerspective.disp.fit();
          m_pjPerspective.updateCamera();          
       }

       //////////////////////////////////////////////////////////////////////
       // Visualization panel
       //////////////////////////////////////////////////////////////////////
       if (source == viewEdgesCB)
       {
          if (viewEdgesCB.getState() == true)
          {
             m_pjPerspective.poly.showPolygons(true);
             m_pjPerspective.poly.update(m_pjPerspective.poly);

          }
          else
          {
             m_pjPerspective.poly.showPolygons(false);
             m_pjPerspective.poly.update(m_pjPerspective.poly);
          };
          return;
       };

       if (source == viewVerticesCB)
       {
          if (viewVerticesCB.getState() == true)
          {
             m_pjPerspective.poly.showVertices(true);
             m_pjPerspective.showVertices = true;
             m_pjPerspective.poly.update(m_pjPerspective.poly);
             m_pjPerspective.shadow.showVertices(true);
             m_pjPerspective.shadow.update(m_pjPerspective.shadow);             
          }
          else
          {
             m_pjPerspective.poly.showVertices(false);
             m_pjPerspective.showVertices = false;             
             m_pjPerspective.poly.update(m_pjPerspective.poly);
             m_pjPerspective.shadow.showVertices(false);
             m_pjPerspective.shadow.update(m_pjPerspective.shadow);                          
          }
          return;
       };
       
       if (source == viewJoiningLinesCB)
       {
          if (viewJoiningLinesCB.getState() == true)
          {
             m_pjPerspective.addGeometry(m_pjPerspective.joininglines);
          }
          else
          {
              m_pjPerspective.removeGeometry(m_pjPerspective.joininglines);
          }
          return;
       };       
    }; // End of itemStateChanged()


    //////////////////////////////////////////////////////////////////////////
    // Handle the proper state of the checkbox items in the model panel
    //////////////////////////////////////////////////////////////////////////
    protected void updateModelSelectors()
    {

    }; // End of updateModelSelectors()


    //////////////////////////////////////////////////////////////////////////
    // Build choice of models within a given category
    //////////////////////////////////////////////////////////////////////////
    protected void buildChoice(String c)
    {
    	poly_name.removeAll();
    	if (c.equals(object_trans[idiom]))
    	{
    		for (int i = 0; i < object.getNumberOfModels(); i++)
    		{
    		   poly_name.add(object.getModel(i).getName(idiom));
    		}
    	}
    	    	
    	if (c.equals(platonic_trans[idiom]))
    	{
    		for (int i = 0; i < platonic.getNumberOfModels(); i++)
    		{
    		   poly_name.add(platonic.getModel(i).getName(idiom));
    		}
    	}

    	if (c.equals(animal_trans[idiom]))
    	{
    		for (int i = 0; i < animal.getNumberOfModels(); i++)
    		{
    		   poly_name.add(animal.getModel(i).getName(idiom));
    		}
    	}    	    	
    }


    //////////////////////////////////////////////////////////////////////////
    // Build database of solids
    //////////////////////////////////////////////////////////////////////////
    protected void buildDataBase()
    {
        int i;
        
        // OBJECT
        i = 0;
        object = new Category(object_trans, 12);

        object.addModel(i, new Model(object_trans,
                new String[]
                {"Ponto",
                 "Point"},
                "jvx",
                "object-point",
                true, true,
                true,
                true));
        object.getModel(i).setDist(83.88919);
        object.getModel(i).showVertexLabels(true);
        object.getModel(i).enableRotation(false);
        i++;
        
        object.addModel(i, new Model(object_trans,
                new String[]
                {"Segmentos (Exemplo 1)",
                 "Segments (Example 1)"},
                "jvx",
                "object-segment-1",
                true, true,
                true,
                true));
        object.getModel(i).setDist(44.75509); 
        object.getModel(i).setZMax(-10.0);
        i++;        

        object.addModel(i, new Model(object_trans,
                new String[]
                {"Segmentos (Exemplo 2)",
                 "Segments (Example 2)"},
                "jvx",
                "object-segment-2",
                true, true,
                true,
                true));
        object.getModel(i).setDist(58.26825);
        object.getModel(i).setVisibility(4);
        object.getModel(i).setZMax(-5.0);        
        i++;        

        object.addModel(i, new Model(object_trans,
                new String[]
                {"Segmentos (Exemplo 3)",
                 "Segments (Example 3)"},
                "jvx",
                "object-segment-3",
                true, true, 
                true,
                true));        
        object.getModel(i).setDist(64.77512);
        object.getModel(i).setZMax(-12.0);        
        i++;
        
        object.addModel(i, new Model(object_trans,
                new String[]
                {"Segmentos (Exemplo 4)",
                 "Segments (Example 4)"},
                "jvx",
                "object-segment-4",
                true, true, 
                true,
                true));        
        object.getModel(i).setDist(44.53393);
        object.getModel(i).setEye(10, 0, -30);
        object.getModel(i).setVisibility(1.60);
        object.getModel(i).setZVal(-5.0);
        object.getModel(i).setZMax(-4.95);        
        i++;
        
        object.addModel(i, new Model(object_trans,
                new String[]
                {"Segmentos (Exemplo 5)",
                 "Segments (Example 5)"},
                "jvx",
                "object-segment-5",
                true, true, 
                true,
                true));        
        object.getModel(i).setDist(44.53393);
        object.getModel(i).setEye(10, 0, -30);
        object.getModel(i).setVisibility(1.60);
        object.getModel(i).setZVal(-5.0);
        object.getModel(i).setZMax(-4.95);        
        i++;
        
        object.addModel(i, new Model(object_trans,
                new String[]
                {"Segmentos (Exemplo 6)",
                 "Segments (Example 6)"},
                "jvx",
                "object-segment-6",
                true, true, 
                true,
                true));        
        object.getModel(i).setDist(64.77512);
        object.getModel(i).setZMax(-10.0);        
        i++;       

        object.addModel(i, new Model(object_trans,
                new String[]
                {"Segmento dividido em duas partes iguais",
                 "Segment divided in two equal parts"},
                "jvx",
                "object-segment-7",
                true, true,
                true,
                true));
        object.getModel(i).setDist(44.75509); 
        object.getModel(i).setZMax(-10.0);
        i++;     

        object.addModel(i, new Model(object_trans,
                new String[]
                {"Segmento dividido na razo urea",
                 "Segment divided in the golden ratio"},
                "jvx",
                "object-segment-8",
                true, true,
                true,
                true));
        object.getModel(i).setDist(44.75509); 
        object.getModel(i).setZMax(-10.0);
        i++;   
        
        object.addModel(i, new Model(object_trans,
                new String[]
                {"Tringulo equiltero",
                 "Equilateral triangle"},
                "jvx",
                "object-triangle-1",
                true, true, 
                true,
                true));
        object.getModel(i).setDist(50.0);   
        object.getModel(i).setZMax(-7.0);        
        i++;        
       
        object.addModel(i, new Model(object_trans,
                new String[]
                {"Crculo",
                 "Circle"},
                "jvx",
                "object-circle",
                true, true, 
                true,
                true));
        object.getModel(i).setDist(50.0);        
        i++;

        object.addModel(i, new Model(object_trans,
                new String[]
                {"MEC",
                 "MEC"},
                "jvx",
                "object-mec",
                false, true, 
                true,
                true));
        object.getModel(i).setDist(50.0);     
        object.getModel(i).setZVal(-10.0);
        i++;
        
        // PLATONIC
        i = 0;
        platonic = new Category(platonic_trans, 5);

        platonic.addModel(i, new Model(platonic_trans,
                new String[]
                {"Tetraedro",
                 "Tetrahedron"},
                "jvx",
                "platonic-tetrahedron",
                true, true,
                true,
                true));
        platonic.getModel(i).setEye(0.0, 0.0, -25.0);
        platonic.getModel(i).setZVal(-15.0); 
        i++;

        platonic.addModel(i, new Model(platonic_trans,
                new String[]
                {"Cubo",
                 "Cube"},
                "jvx",
                "platonic-cube",
                true, true,
                true,
                true));
        platonic.getModel(i).setEye(0.0, 0.0, -25.0);
        platonic.getModel(i).setZVal(-15.0);        
        i++;

        platonic.addModel(i, new Model(platonic_trans,
                new String[]
                {"Octaedro",
                 "Octahedron"},
                "jvx",
                "platonic-octahedron",
                true, true,
                true,
                true));
        platonic.getModel(i).setEye(0.0, 0.0, -25.0);
        platonic.getModel(i).setZVal(-15.0); 
        platonic.getModel(i).setZMax(-10.0);
        i++;

        platonic.addModel(i, new Model(platonic_trans,
                new String[]
                {"Dodecaedro",
                 "Dodecahedron"},
                "jvx",
                "platonic-dodecahedron",
                true, true, 
                true,
                true));
        platonic.getModel(i).setEye(0.0, 0.0, -25.0);
        platonic.getModel(i).setZVal(-15.0);
        platonic.getModel(i).setZMax(-10.0);        
        i++;

        platonic.addModel(i, new Model(platonic_trans,
                new String[]
                {"Icosaedro",
                 "Icosahedron"},
                "jvx",
                "platonic-icosahedron",
                true, true, 
                true,
                true));
        platonic.getModel(i).setEye(0.0, 0.0, -25.0);
        platonic.getModel(i).setZVal(-15.0);
        platonic.getModel(i).setZMax(-10.0);        
        i++;

        // ANIMAL
        i = 0;
        animal = new Category(animal_trans, 2);

        animal.addModel(i, new Model(animal_trans,
               new String[]
               {"Cavalo",
                "Horse"},
               "jvx",
               "animal-horse",
               false, true, 
               false,
               true));
        animal.getModel(i).setScale(12.0);
        animal.getModel(i).setDist(55.0);   
        animal.getModel(i).setZVal(-12.0);  
        animal.getModel(i).enablePosition(false);
        i++;

        animal.addModel(i, new Model(animal_trans,
               new String[]
               {"Dromedrio",
                "Dromedary"},
               "jvx",
               "animal-dromedary",
               false, true, 
               false,
               true));
        animal.getModel(i).setScale(12.0);
        animal.getModel(i).setDist(55.0);   
        animal.getModel(i).setZVal(-12.0);        
        animal.getModel(i).enablePosition(false);        
        i++;
    }
}; // End of PjPerspective_IP class
