import jv.geom.*;
import jv.object.PsObject;
import jv.objectGui.PsImage;
import jv.project.PgGeometryIf;
import jv.project.PjProject;
import jv.project.PvCameraIf;
import jv.project.PvDisplayIf;
import jv.vecmath.PdVector;
import jvx.geom.PwRefineElementSet;

public class PjTexturePlane extends PjProject implements Runnable
{
    protected String imageFileName; 	
    protected double imageWidth;
    protected double imageHeight;
    protected int number_of_faces = 1;

    protected String[] m_texImage;
    protected PsImage m_image[];
    protected PgElementSet m_face[];
    
    protected PwRefineElementSet m_refine;

    protected double dist;
    
    
    public PjTexturePlane(String input_imageFileName,
    		              double input_imageWidth,
    		              double input_imageHeight,
			      double input_dist)
    {
        super("Textured Plane");
        
    	imageFileName = input_imageFileName;
    	imageWidth = input_imageWidth;
    	imageHeight = input_imageHeight;

	dist = input_dist;
    	
    	m_refine = new PwRefineElementSet();
    	
    	m_texImage = new String[number_of_faces];
    	m_texImage[0] = imageFileName;

        m_image = new PsImage[number_of_faces];
        m_face = new PgElementSet[number_of_faces];
        int i = 0;
        do
        {
            m_face[i] = new PgElementSet(3);
        }            
        while(++i < number_of_faces);
        if (getClass() == (PjTexturePlane.class))
        {
            init();
        }
        
 
    }

    public void start()
    {
        Thread thread = new Thread(this, "JavaView: loading texture images");
        thread.start();
        super.start();
        
        PvDisplayIf disp = getDisplay(); 
        PvCameraIf cam = disp.getCamera();   
        
    	cam.setViewDir(new PdVector(0, 0, -1));
	cam.setInterest(new PdVector(0, 0, 0));
    	cam.setRoll(0);
	cam.setDist(dist);
    	cam.update(cam);
	disp.update(disp);
    	// disp.fit();
    }

    public void run()
    {
        int i = 0;
        do
        {
            m_image[i] = new PsImage(m_texImage[i]);
            if(m_image[i] != null)
            {
                m_image[i].loadImage();
                m_face[i].makeQuadrElementTexture(2, 2);
                PgTexture pgtexture = new PgTexture();
                pgtexture.setImage(m_image[i].getImage());
                m_face[i].setTexture(pgtexture);
                m_face[i].showElementTexture(true);
                m_face[i].update(m_face[i]);
                m_face[i].showEdges(false);
                m_refine.setGeometry((PgElementSet) m_face[i]);
                m_refine.refineQuadrisection();
                m_refine.refineQuadrisection();
                m_refine.refineQuadrisection();
                m_refine.refineQuadrisection();
                m_refine.refineQuadrisection();
                m_face[i].update(m_face[i]);
            }
        } while(++i < number_of_faces);

        PvDisplayIf disp = getDisplay(); 
	if (disp != null)
        {
           PvCameraIf cam = disp.getCamera();   
        
    	   cam.setViewDir(new PdVector(0, 0, -1));
	   cam.setInterest(new PdVector(0, 0, 0));
    	   cam.setRoll(0);
	   cam.setDist(dist);
    	   cam.update(cam);
	   disp.update(disp);
	};
    }

    public void init()
    {
        super.init();
        PdVector apdvector[] = new PdVector[4];
        double a = imageWidth/2.0;
        double b = imageHeight/2.0;
        apdvector[0] = new PdVector(-a, -b, 0);
        apdvector[1] = new PdVector( a, -b, 0);
        apdvector[2] = new PdVector( a,  b, 0);
        apdvector[3] = new PdVector(-a,  b, 0);
        m_face[0].buildQuadr(apdvector, 2, 2);
        m_face[0].setName("Top");
        int i = 0;
        do
        {
            m_face[i].makeVertexNormals();
            m_face[i].makeElementNormals();
            m_face[i].showBackface(true);
            addGeometry(m_face[i]);
        } while(++i < number_of_faces);
        selectGeometry(m_face[0]);        
    }
}
