//////////////////////////////////////////////////////////////////////////////
// Plane Image class
//
// Last update: 20/03/2005
//////////////////////////////////////////////////////////////////////////////

import java.applet.Applet;
import java.awt.BorderLayout;
import java.awt.Color;

import jv.viewer.PvDisplay;
import jv.viewer.PvViewer;
import jv.geom.PgElementSet;
import jv.vecmath.PdVector;

public class PlaneImage extends Applet
{
    // Builds a cube given its center and size
    public PgElementSet buildCube(double xcenter,
                                  double ycenter,
                                  double zcenter,
                                  double size)
    {
       // Corner coordinates
       final double xmin = xcenter - size/2.0;
       final double ymin = ycenter - size/2.0;
       final double zmin = zcenter - size/2.0;

       PgElementSet cube = new PgElementSet(3); // This is a 3D object

       cube.setNumElements(6); // Our cube has 6 faces
       cube.setNumVertices(8); // Our cube has 8 vertices

       // Setup vertices coordinates
       cube.setVertex(0, new PdVector(xmin, ymin, zmin));
       cube.setVertex(1, new PdVector(xmin + size, ymin, zmin));
       cube.setVertex(2, new PdVector(xmin, ymin + size, zmin));
       cube.setVertex(3, new PdVector(xmin + size, ymin + size, zmin));
       cube.setVertex(4, new PdVector(xmin, ymin, zmin + size));
       cube.setVertex(5, new PdVector(xmin + size, ymin, zmin + size));
       cube.setVertex(6, new PdVector(xmin, ymin + size, zmin + size));
       cube.setVertex(7, new PdVector(xmin + size, ymin + size, zmin + size));

       // Setup faces specifying vertices id order
       cube.setElement(0, 0, 1, 3, 2);
       cube.setElement(1, 0, 1, 5, 4);
       cube.setElement(2, 1, 3, 7, 5);
       cube.setElement(3, 3, 2, 6, 7);
       cube.setElement(4, 0, 2, 6, 4);
       cube.setElement(5, 4, 5, 7, 6);

       return(cube);
   }; // End of buildCube()

   // This procedure is executed when the applet starts
   public void init()
   {
      // Viewer
      PvViewer viewer = new PvViewer(this, null);

      // Display
      PvDisplay disp = new PvDisplay();

      // Axes
      disp.showAxes(false);

      // Bounding box
      disp.showBndBox(true);

      // Background color
      disp.setBackgroundColor(Color.WHITE);

      // Define two cubes
      PgElementSet cube1 = buildCube(0, 0, 0, 1);
      PgElementSet cube2 = buildCube(0, 0, 0, 0.5);

      // Cube 1 properties
      cube1.setGlobalEdgeColor(Color.BLACK);
      cube1.setGlobalVertexColor(Color.RED);
      cube1.setGlobalVertexSize(2.0);
      cube1.showElementColors(true);
      cube1.setElementColor(5, Color.RED);

      // Cube 2 properties
      cube1.setTransparency(0.5);
      cube1.showVertices(false);
      cube1.showTransparency(true);
      cube1.showElementColors(true);
      cube1.showElements(true);
      cube1.showEdges(true);

      // Register cubes in the display
      disp.addGeometry(cube1);
      disp.addGeometry(cube2);

      // Applet layout, quite simple now
      setLayout(new BorderLayout());
      add(disp, BorderLayout.CENTER);
    }; // End of init()
};
